@@ -8,7 +8,7 @@
 
 ;; sb-xref-for-internals is actively harmful to tree-shaking.
 ;; Remove some symbols to make the hide-packages test pass.
-#+sb-xref-for-internals
+#+(and sb-xref-for-internals (not sb-devel))
 (progn
   (fmakunbound 'sb-kernel::type-class-fun-slot)
   (fmakunbound 'sb-kernel::new-ctype))
@@ -99,32 +99,9 @@
        (removable-features
         (append non-target-features public-features)))
   (defconstant sb-impl:+internal-features+
-    ;;; Well, who would have guessed that our internal features list would nicely
-    ;;; repair damage induced by ASDF, namely: ASDF removes features when loaded.
-    ;;; Take a look at (DEFUN DETECT-OS) in uiop.lisp if you don't believe it,
-    ;;; and watch it in action after (require "ASDF") -
-    ;;;
-    ;;; * *FEATURES* =>
-    ;;; (:X86-64 :64-BIT :ANSI-CL :COMMON-LISP :ELF :GENCGC :HAIKU :IEEE-FLOATING-POINT
-    ;;; :LITTLE-ENDIAN :PACKAGE-LOCAL-NICKNAMES :SB-LDB
-    ;;; :SB-PACKAGE-LOCKS :SB-UNICODE :SBCL :UNIX)
-    ;;;
-    ;;; * (require :asdf)
-    ;;; ("ASDF" "asdf" "UIOP" "uiop")
-    ;;; * *FEATURES*
-    ;;; (:ASDF3.3 :ASDF3.2 :ASDF3.1 :ASDF3 :ASDF2 :ASDF :OS-UNIX
-    ;;; :NON-BASE-CHARS-EXIST-P :ASDF-UNICODE :X86-64 :64-BIT :ANSI-CL :COMMON-LISP
-    ;;; :ELF :GENCGC :IEEE-FLOATING-POINT :LITTLE-ENDIAN :PACKAGE-LOCAL-NICKNAMES
-    ;;; :SB-LDB :SB-PACKAGE-LOCKS :SB-UNICODE :SBCL :UNIX)
-    ;;;
-    ;;; So, what the heck happened to :HAIKU? It's gone.
-    ;;; Well this is pure evil. Just madness.
-    ;;; However, by stashing an extra copy of :HAIKU in the internal feature list,
-    ;;; we can stuff it back in because our contrib builder first loads ASDF
-    ;;; and then rebinds *FEATURES* with the union of the internal ones.
-    (append #+haiku '(:haiku)
             (remove-if (lambda (x) (member x #+sb-devel public-features
-                                             #-sb-devel removable-features)) *features*)))
+                                             #-sb-devel removable-features))
+                       *features*))
   (setq *features* (remove-if-not (lambda (x) (member x public-features))
                                   *features*)))
 
@@ -137,7 +114,7 @@
 ;;; uninterned.
 ;;; Additionally, you can specify an arbitrary way to destroy
 ;;; random bootstrap stuff on per-package basis.
-(defun !unintern-init-only-stuff (&aux result)
+(defun !unintern-init-only-stuff ()
   (dolist (package (list-all-packages))
     (sb-int:awhen (find-symbol "!REMOVE-BOOTSTRAP-SYMBOLS" package)
       (funcall sb-int:it)))
@@ -172,9 +149,13 @@
                                       (and (consp x) (uninternable-p (car x))))
                                     (dd-constructors dd))))))
              (classoid-subclasses (find-classoid t)))
-    ;; PATHNAME is not a structure-classoid
-    (setf (sb-kernel:dd-constructors (sb-kernel:find-defstruct-description 'pathname))
-          nil)
+
+    (loop for type in '(pathname ;; PATHNAME is not a structure-classoid
+                        sb-c:storage-class
+                        sb-c:storage-base)
+          do
+          (setf (sb-kernel:dd-constructors (sb-kernel:find-defstruct-description type))
+                nil))
     ;; Todo: perform one pass, then a full GC, then a final pass to confirm
     ;; it worked. It should be an error if any uninternable symbols remain,
     ;; but at present there are about 7 symbols with referrers.
@@ -209,7 +190,7 @@
     (setf (sb-c::vop-parse-body v) nil))
   ;; Used for inheriting from other VOPs, not needed in the target.
   (setf sb-c::*backend-parsed-vops* (make-hash-table))
-  result)
+  nil)
 
 
 ;;; Check for potentially bad format-control strings
@@ -296,10 +277,10 @@ Please check that all strings which were not recognizable to the compiler
     (when (typep info 'sb-c::compiled-debug-info)
       (let ((map (sb-c::compiled-debug-info-fun-map info)))
         (when (typep map '(simple-array (unsigned-byte 8) (*)))
-          (sb-alien:with-alien ((compress-vector (function int (* char) size-t) :extern "compress_vector"))
+          (sb-alien:with-alien ((compress-vector (function int unsigned size-t) :extern))
             (sb-sys:with-pinned-objects (map)
               (sb-alien:alien-funcall compress-vector
-                                      (sb-sys:int-sap (sb-kernel:get-lisp-obj-address map))
+                                      (sb-kernel:get-lisp-obj-address map)
                                       (length map)))))))))
 (progn
   ;; Remove source forms of compiled-to-memory lambda expressions.
@@ -537,6 +518,23 @@ Please check that all strings which were not recognizable to the compiler
 
 (scan-format-control-strings)
 
+(macrolet ((def-backward-compatible-sb-c-specials (pairs) ; for UIOP + ASDF
+             `(progn
+                ,@(mapcar (lambda (pair)
+                            `(define-symbol-macro ,(car pair)
+                                 (,(sb-int:package-symbolicate "SB-C" "CU-" (cdr pair) "-COUNT")
+                                   sb-c::*compilation-unit*)))
+                          pairs))))
+  ;; coece to a strict boolean
+  (define-symbol-macro sb-c::*in-compilation-unit* (not (null sb-c::*compilation-unit*)))
+  ;; the "specials" are all SETFable when and only when *IN-COMPILATION-UNIT* is T
+  (def-backward-compatible-sb-c-specials
+      sb-c::((*aborted-compilation-unit-count* . "ABORTED")
+             (*compiler-error-count* . "ERROR")
+             (*compiler-warning-count*  . "WARNING")
+             (*compiler-style-warning-count* . "STYLE-WARNING")
+             (*compiler-note-count* . "NOTE"))))
+
 #+sb-devel
 (rename-package "COMMON-LISP" "COMMON-LISP" '("SB-XC" "CL"))
 
