@@ -415,7 +415,9 @@
      (typep (- d) '(integer -47727025476642942 -2593702250735)))
    null))
 
-(with-test (:name :signed-byte-8-p-unsigned)
+(with-test (:name :signed-byte-8-p-unsigned
+                  ;; these lack the necessary RANGE<= vop
+                  :fails-on (:or :mips :ppc :ppc64 :sparc :riscv))
   (checked-compile
    `(lambda (a)
       (declare (type (simple-array sb-vm:word (*)) a)
@@ -582,3 +584,65 @@
               ((unsigned-byte 8) a))
      (floor a b))
    (values (integer -255 0) (integer -9 0) &optional)))
+
+(with-test (:name :logbitp-on-integers)
+  (assert (not (ctu:ir1-named-calls `(lambda (x)
+                                       (logbitp 20 x))))))
+(with-test (:name :bt-negative-bit)
+  (checked-compile-and-assert
+   ()
+   `(lambda (c)
+     (declare ((signed-byte 64) c))
+     (logtest c -2199023255553))
+   ((-2049990302793354782) t)
+   ((0) nil)
+   (((ash 1 41)) nil))
+  (checked-compile-and-assert
+   ()
+   `(lambda (b)
+      (declare (fixnum b))
+      (logior b -4611686018427387905))
+   ((-6) -1)))
+
+(with-test (:name :float-cmp)
+  (checked-compile-and-assert
+      ()
+      `(lambda (a b)
+         (declare ((unsigned-byte 20) a)
+                  (float b))
+         (< a b))
+    ((6 4.0) nil)
+    ((1 1.1) t)))
+
+(with-test (:name :complex+non-complex-type)
+  (assert-type
+   (lambda (a)
+     (+ a #c(1.0 3.0)))
+   (or (complex single-float) (complex double-float)))
+  (assert-type
+   (lambda (a)
+     (* a #c(1d0 0d0)))
+    (complex double-float)))
+
+(with-test (:name :bignum-ash-modarith)
+  (checked-compile-and-assert
+      ()
+      `(lambda (a)
+         (declare (bignum a))
+         (logand (ash a -1) 1))
+    (((expt 2 129)) 0)
+    (((+ (expt 2 129) 2)) 1)))
+
+(with-test (:name :two-fixnum-eq-mask)
+  (checked-compile-and-assert
+      ()
+      `(lambda (x)
+         (declare (fixnum x))
+         (typep x '(member -1 ,most-positive-fixnum)))
+    ((-1) t)
+    ((most-positive-fixnum) t)
+    ((1) nil)
+    ((0) nil)
+    ((most-negative-fixnum) nil)
+    (((1+ most-negative-fixnum)) nil)
+    (((1- most-positive-fixnum)) nil)))
